from django.conf.urls.defaults import *

urlpatterns = patterns('peavy.views',
    url(r'^$', 'dashboard', name='dashboard'),
    url(r'^debug/(?P<record_id>[^/]+)/$', 'debug_page', name='debug_page'),
)




from django.core.signals import got_request_exception
from django.db import transaction

@transaction.commit_on_success
def exception_handler(request=None, **kwargs):
    logger = logging.getLogger('peavy.signals.exception_handler')

    if transaction.is_dirty():
        transaction.rollback()

    logger.exception('Exception: {0}'.format(exc_type), exc_info=True)

got_request_exception.connect(exception_handler)




import re

from django.conf import settings
from django import http
from django import template
from django.utils.html import fix_ampersands
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

register = template.Library()

PAGE_RE = re.compile("[\?&]page=\d+")
class PageNavigatorNode(template.Node):
    """
    Renders page navigation markup for a page of a django.core.paginator.Paginator.
    """
    def __init__(self, page, base_url):
        self.page = page
        self.base_url = base_url
    
    def render(self, context):
        page = template.Variable(self.page).resolve(context)
        url = template.Variable(self.base_url).resolve(context)
        url = PAGE_RE.sub("", url)
        url = url.replace("%", "%%")
        
        pages = page.paginator.num_pages

        page_link = u'<a class="pagelink %%s" href="%s%spage=%%s"><img src="%s%%s" alt=""></a>' % (url, url.find("?") == -1 and  "?" or "&", settings.STATIC_URL)
        first = getattr(settings, "PAGE_ICON_FIRST", "peavy/img/first.png")
        prev = getattr(settings, "PAGE_ICON_PREV", "peavy/img/prev.png")
        next = getattr(settings, "PAGE_ICON_NEXT", "peavy/img/next.png")
        last = getattr(settings, "PAGE_ICON_LAST", "peavy/img/last.png")

        output = ""
        if page.paginator.num_pages > 1:
            output = u'<div class="page_navigator">%s: ' % _(u"Page")
            output += page_link % ("first", 1, first)
            if page.has_previous:
                output += page_link % ("previous", page.previous_page_number(), prev)
            output += " %s / %s " % (page.number, pages)
            if page.has_next:
                output += page_link % ("next", page.next_page_number(), next)
            output += page_link % ("last", pages, last)                
            output += "</div>"
        return mark_safe(fix_ampersands(output))

@register.tag
def pagenavigator(parser, argument_string):
    argv = argument_string.split_contents()
    argc = len(argv)
    if argc == 3:
        return PageNavigatorNode(argv[1], argv[2])
    
    raise template.TemplateSyntaxError("The %s tag requires a paginator page and a base URL." % argv[0])

class QueryStringNode(template.Node):
    """
    Updates the current query string of the request with the supplied
    parameters and values.

    Values are first looked up as variables in the request context; if the
    variable is not found they'll be used as given.

    You can add or delete specific values, or by passing an empty value, remove
    all values for a parameter. The order in which you pass the parameters is
    the order in which the operations will be applied to the query string.

    Add several parameters:

        {% query_string "level=ERROR" "level=WARN" "logger=record.logger" %}

    Delete a value, if present:

        {% query_string "level-=record.level" %}

    Clear all values for a certain parameter, then add one for it:

        {% query_string "level=" "level=CRITICAL" %}

    """
    parameters = []

    def __init__(self, parameters):
        self.parameters = parameters
    
    def render(self, context):

        operations = []

        for p in self.parameters:
            if '-=' in p:
                # handle requests to delete a specific value
                k, v = p.split('-=', 1)
                operations.append(('delete', k, v))
            else:
                k, v = p.split('=', 1)
                if not v:
                    # handle requests to remove all values
                    operations.append(('clear', k, v))
                else:
                    # add a value for the key
                    operations.append(('add', k, v))
        request = template.Variable('request').resolve(context)

        qs = request.GET.copy()
        for operation, k, v in operations: 
            try:
                k = template.Variable(k).resolve(context)
            except template.VariableDoesNotExist, e:
                # OK, we'll use the key as given
                pass

            if operation == 'clear':
                if k in qs:
                    del qs[k]
                continue

            try:
                v = template.Variable(v).resolve(context)
            except template.VariableDoesNotExist, e:
                # OK, we'll use the value as given
                pass

            if operation == 'add':
                values = qs.getlist(k)
                if v not in values:
                    # repeating parameters is OK, but there's no point repeating
                    # the same values
                    values.append(v)
                    qs.setlist(k, values)

            elif operation == 'delete':
                if k in qs:
                    values = qs.getlist(k)
                    qs.setlist(k, [val for val in values if val != v])

        query_string = qs.urlencode()
        if query_string:
            return '?' + query_string
        else:
            return ''

@register.tag
def query_string(parser, argument_string):
    argv = argument_string.split_contents()
    argc = len(argv)

    return QueryStringNode(argv[1:])



import logging

from django.db import transaction
from django.test import TestCase
from django.test.client import Client

from peavy.models import LogRecord
from peavy_demo.models import Quote

class TransactionTest(TestCase):

    def testTransactionSeparation(self):
        """
        Under the demo application's settings, the app and peavy both log to
        the same database, but on different connections. This test verifies
        that this means a rollback of an application write does not undo
        the logging made in that code.
        """
        logger = logging.getLogger('peavy.tests.TransactionTest.testTransactionSeparation')

        try:
            with transaction.commit_on_success():

                Quote.objects.create(
                    submitter='Anonymous Browncoat',
                    show='Firefly',
                    character='Wash',
                    text='Ah, curse your sudden but inevitable betrayal!'
                )

                logger.info('Someone left a quote!')

                # peavy has just committed the log entry; now raise an
                # exception to undo the quote creation
                raise ValueError, 'As in: Fox miscalculated the value of Firefly.'

        except ValueError, e:
            pass
        
        self.assertTrue(Quote.objects.count() == 0, 'The quote was not rolled back.')
        self.assertTrue(LogRecord.objects.count() == 1, 'The log entry was not made.')



import logging

from peavy.middleware import RequestLoggingMiddleware

class BasicFilter(logging.Filter):
    """
    Adds basic request info (unique ID, user, client IP) into the log record.
    """

    def filter(self, record):
        request = RequestLoggingMiddleware.context.request

        record.uuid = getattr(request, "uuid", "?")
        record.user = getattr(request, "user", "?")
        meta = getattr(request, "META", {})
        record.client_ip = meta.get("REMOTE_ADDR", "?")
        return True

class MetaFilter(logging.Filter):
    """
    Adds all the request metadata to the log record.
    """

    def filter(self, record):
        request = RequestLoggingMiddleware.context.request

        meta = getattr(request, "META", {})
        for key, value in meta.items():
            if hasattr(record, key):
                continue
            setattr(record, "META_" + key, value)

        return True




import logging
import traceback

class ExceptionLoggingHandler(logging.Handler):
    """
    A handler that collects exception information, including a stack trace and
    a full Django error page.
    """

    def get_debug_page(self, record, request):
        """
        Given a logging.LogRecord and a Django request that encountered an
        exception, create a Django server error debugging page.
        """

        from django.views.debug import ExceptionReporter

        debug_page = ''
        if record.exc_info:
            exc_info = record.exc_info
            reporter = ExceptionReporter(request, *exc_info)
            debug_page = reporter.get_traceback_html() or ''
        return debug_page

    def get_stack_trace(self, record, request):
        """
        Given a logging.LogRecord and a Django request that encountered an
        exception, collect a stack trace.
        """

        from django.views.debug import ExceptionReporter

        stack_trace = ''
        if record.exc_info:
            exc_info = record.exc_info
            reporter = ExceptionReporter(request, *exc_info)
            stack_trace = '\n'.join(
                traceback.format_exception(*record.exc_info)
            )
        return stack_trace

class DjangoDBHandler(ExceptionLoggingHandler):
    """
    A handler that stores log records in a Django model.
    """
    
    def emit(self, record):
        from django.db import transaction

        from peavy.middleware import RequestLoggingMiddleware
        from peavy.models import LogRecord

        self.format(record)

        request = getattr(RequestLoggingMiddleware.context, 'request', None)

        stack_trace = self.get_stack_trace(record, request)
        debug_page = self.get_debug_page(record, request)

        from django.conf import settings
        db_name = getattr(settings, 'PEAVY_DATABASE_NAME', 'peavy')

        with transaction.commit_on_success(using=db_name):
            LogRecord.objects.using(db_name).create(
                logger = record.name,
                level = record.levelname,
                message = record.msg,
                uuid = getattr(record, 'uuid', '?'),
                client_ip = getattr(record, 'client_ip', '?'),
                user = getattr(record, 'user', '?'),
                stack_trace = stack_trace,
                debug_page = debug_page
            )

class AdminEmailHandler(ExceptionLoggingHandler):
    """
    A handler that sends an abridged error notification to settings.ADMINS; the
    email includes the traceback and a link to the dashboard for reviewing the
    request's log records, but no sensitive data like request.POST.
    """

    def emit(self, record):
        from peavy.middleware import RequestLoggingMiddleware
        from django.core import mail
        from django.core.urlresolvers import reverse
        from django.contrib.sites.models import Site

        subject = '%s: %s' % (
            record.levelname,
            record.msg
        )

        try:
            request = getattr(RequestLoggingMiddleware.context, 'request', None)
            site = Site.objects.get_current()
            request_link = (
                "View the log entries for this request:\n\n"
                "https://%s%s?request_id=%s\n" % (
                    site.domain,
                    reverse('peavy:dashboard'),
                    record.uuid
                )
            )

        except Exception, e:
            print e
            request = None
            request_link = "Link to request log records unavailable."

        stack_trace = ""
        if record.exc_info:
            stack_trace = self.get_stack_trace(record, request)

        message = "%s\n\n%s" % (request_link, stack_trace)

        mail.mail_admins(
            subject,
            message,
            fail_silently=True
        )



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding index on 'LogRecord', fields ['timestamp']
        db.create_index('peavy_logrecord', ['timestamp'])


    def backwards(self, orm):
        
        # Removing index on 'LogRecord', fields ['timestamp']
        db.delete_index('peavy_logrecord', ['timestamp'])


    models = {
        'peavy.logrecord': {
            'Meta': {'ordering': "('-timestamp',)", 'object_name': 'LogRecord'},
            'application': ('django.db.models.fields.CharField', [], {'default': "'sandbox'", 'max_length': '256', 'db_index': 'True'}),
            'client_ip': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '128', 'blank': 'True'}),
            'debug_page': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.CharField', [], {'max_length': '32', 'db_index': 'True'}),
            'logger': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'db_index': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'origin_server': ('django.db.models.fields.CharField', [], {'default': "'kaze.jkcl.local'", 'max_length': '256', 'db_index': 'True'}),
            'stack_trace': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'db_index': 'True'}),
            'user': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '256', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '256', 'blank': 'True'})
        }
    }

    complete_apps = ['peavy']



from django.utils.translation import gettext_lazy as _

from django.contrib import admin

from peavy.models import LogRecord

class LogRecordAdmin(admin.ModelAdmin):
    list_display = ('timestamp', 'application', 'origin_server', 'client_ip', 'user', 'logger', 'level', 'message')
    list_filter = ('application', 'origin_server', 'user', 'logger', 'level')
    search_fields = ('application', 'origin_server', 'client_ip', 'user', 'logger', 'level', 'message')

    fieldsets = (
        (
            _('Server details'),
            {
                'fields': (
                    'timestamp',
                    'application',
                    'origin_server'
                )
            }
        ),
        (
            _('Request details'),
            {
                'fields': (
                    'client_ip',
                    'user',
                )
            }
        ),
        (
            _('Logging details'),
            {
                'fields': (
                    'logger',
                    'level',
                    'message'
                )
            }
        ),
        (
            _('Exception details'),
            {
                'fields': (
                    'stack_trace',
                    'debug_page'
                )
            }
        )
    )

    readonly_fields = LogRecord._meta.get_all_field_names()

admin.site.register(LogRecord, LogRecordAdmin)



from threading import local
import uuid

class RequestThreadlocal(local):
    request = object()

class RequestLoggingMiddleware(object):
    """
    RequestLoggingMiddleware enables better logging of Django requests.

    It assigns a unique ID to each request, using the Python uuid module, and
    keeps a threadlocal reference to the request that logging filters or
    adapters can use to access request attributes.

    It should be installed first in your MIDDLEWARE_CLASSES setting, so that
    the request information is available for logging as early as possible.
    """
    
    context = RequestThreadlocal()

    def process_request(self, request):
        request.uuid = uuid.uuid4().hex
        self.context.request = request
        return None



class DjangoDBRouter(object):
    """
    Routes all database operations on peavy models to settings.PEAVY_DATABASE_NAME.

    Peavy has been tested with multiple databases, but South migrations will
    not work. You'll need to manage the tables yourself with syncdb. You should
    be able to make South ignore peavy with this in your settings.py:

        SOUTH_MIGRATION_MODULES = {
            'peavy': 'ignore',
        }

    See:

        http://south.aeracode.org/docs/settings.html#south-migration-modules
        http://south.aeracode.org/ticket/370

    Alternatively, you can fake Django out, by creating a second entry for
    peavy in settings.DATABASES that uses the same database name, user, etc. as
    default. This will allow peavy to work in a separate connection with its
    own transactions. And as a bonus, the South migrations will work.

    """

    def db_for_read(self, model, **hints):
        from django.conf import settings
        db_name = getattr(settings, 'PEAVY_DATABASE_NAME', 'peavy')

        db = None
        if model._meta.app_label == 'peavy':
            db = db_name
        return db

    def db_for_write(self, model, **hints):
        from django.conf import settings
        db_name = getattr(settings, 'PEAVY_DATABASE_NAME', 'peavy')

        db = None
        if model._meta.app_label == 'peavy':
            db = db_name
        return db

    def allow_relation(self, obj1, obj2, **hints):
        allow = None

        if obj1._meta.app_label == 'peavy' and obj2._meta.app_label == 'peavy':
            allow = True
        return allow




import sys
from setuptools import setup, find_packages

peavy = __import__('peavy')

with open('README.rst') as file:
    long_description = file.read()

setup(
    author = 'Fairview Computing LLC',
    author_email = 'john@fairviewcomputing.com',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Logging',
    ],
    description = peavy.__doc__,
    long_description=long_description,
    download_url='http://github.com/fairview/django-peavy/downloads',
    install_requires = [
        'Django>=1.3',
        'South>=0.7.2'
    ],
    license = "MIT License",
    name = 'django-peavy',
    packages = [
        'peavy',
        'peavy.migrations',
        'peavy.templatetags',
    ],
    package_data = {
        'peavy': [
            'README.rst',
            'LICENSE.txt',
            'templates/*/*.html',
           'static/*/*/*',
        ],
    },
    tests_require = [
        'Django>=1.3',
        'South>=0.7.2',
        'psycopg2==2.4.1', # 2.4.2 causes trouble with Django 1.3(.1) unit tests
    ],
    test_suite='tests.main',
    url = 'http://github.com/fairview/django-peavy',
    version = peavy.get_version(),
    zip_safe = True,
)



import datetime
import socket

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

class LogRecord(models.Model):
    """
    A simple bucket for the usual logging info (logger, level, message) plus a
    unique request ID for tracking all the log records made in a given request,
    the server logging the request, the client's IP address, the name of the
    application, and for requests with errors, a stack trace and a copy of
    Django's server error page.
    """
    timestamp = models.DateTimeField(db_index=True, default=datetime.datetime.now)

    application = models.CharField(
        max_length = 256,
        default = getattr(
            settings,
            'PEAVY_APP_NAME',
            settings.ROOT_URLCONF.split('.')[0]
        ),
        help_text = _("The application logging this record."),
        db_index = True
    )

    origin_server = models.CharField(
        max_length = 256,
        help_text = _("The server logging this record."),
        default = socket.gethostname,
        db_index = True
    )

    client_ip = models.CharField(
        max_length = 128,
        help_text = _("The IP address of the client making the request."),
        blank = True,
        db_index = True
    )

    user = models.CharField(
        max_length = 256,
        help_text = _("The username of the user making the request in which this record was logged."),
        blank = True,
        db_index = True
    )

    uuid = models.CharField(
        max_length = 256,
        help_text = _("The UUID of the Django request in which this record was logged."),
        blank = True,
        db_index = True
    )

    logger = models.CharField(
        max_length = 1024,
        help_text = _("The name of the logger of the record."),
        db_index = True
    )

    level = models.CharField(
        max_length = 32,
        help_text = _("The level of the log record (DEBUG, INFO...)"),
        db_index = True
    )

    message = models.TextField(db_index=True)
    stack_trace = models.TextField(blank=True)
    debug_page = models.TextField(blank=True)

    class Meta:
        ordering = ('-timestamp',)
        permissions = (
            ("view_logs", "Can view log records"),
        )

    def __unicode__(self):
        return unicode(self.pk)




import re

from django.conf import settings
from django.contrib.auth.decorators import permission_required, user_passes_test
from django.core.paginator import EmptyPage, Paginator
from django.core.urlresolvers import reverse
from django.db.models import Count, Sum, Q
from django import http
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _

from peavy.models import LogRecord

@permission_required('peavy.view_logs')
def dashboard(request):
    """
    The main view: all the logs, with filters and pagination.
    """
    
    records = LogRecord.objects.all()

    applications = request.GET.getlist('application')
    if applications:
        records = records.filter(application__in=applications)

    client_ips = request.GET.getlist('client_ip')
    if client_ips:
        records = records.filter(client_ip__in=client_ips)

    levels = request.GET.getlist('level')
    if levels:
        records = records.filter(level__in=levels)

    loggers = request.GET.getlist('logger')
    if loggers:
        records = records.filter(logger__in=loggers)

    origin_servers = request.GET.getlist('origin_server')
    if origin_servers:
        records = records.filter(origin_server__in=origin_servers)

    request_ids = request.GET.getlist('request_id')
    if request_ids:
        records = records.filter(uuid__in=request_ids)

    users = request.GET.getlist('username')
    if users:
        records = records.filter(user__in=users)

    message_filters = request.GET.getlist('message')
    if message_filters:
        message_query = None
        for term in message_filters:
            if not term:
                continue
            if message_query is None:
                message_query = Q(message__iregex=term)
            else:
                message_query &= Q(message__iregex=term)
        if message_query is not None:
            records = records.filter(message_query)

    page_number = int(request.GET.get('page', 1))
    count = int(request.GET.get('count', 20))

    paginator = Paginator(object_list=records, per_page=count, allow_empty_first_page=True)

    if page_number < 1:
        redirect = re.sub('page=\d+', 'page=%s' % paginator.num_pages, request.get_full_path())
        return http.HttpResponseRedirect(redirect)
    if page_number > paginator.num_pages:
        redirect = re.sub('page=\d+', 'page=1', request.get_full_path())
        return http.HttpResponseRedirect(redirect)

    records = paginator.page(page_number)

    data = {
        "records": records,
    }

    return render_to_response(
        'peavy/dashboard.html',
        data,
        context_instance = RequestContext(request)
    )

@user_passes_test(lambda u: u.is_superuser)
def debug_page(request, record_id):
    """
    For log records with exception information, return the copy of Django's
    error page for the request. As this can contain sensitive information like
    passwords in POST data, restrict the view to superusers.
    """

    record = get_object_or_404(LogRecord, pk=record_id)
    return http.HttpResponse(record.debug_page)
